use std::path::PathBuf;

use gtk::prelude::*;
use relm4::prelude::*;
use relm4::{
    gtk, Component, ComponentController, ComponentParts, ComponentSender, SimpleComponent,
};
use walkdir::WalkDir;

pub struct SheetModel {
    path: PathBuf,
}

#[derive(Debug)]
pub enum SheetModelInput {
    OnClicked,
}

#[derive(Debug)]
pub enum SheetPressedMessage {
    SheetPressed(PathBuf),
}

#[relm4::component(pub)]
impl SimpleComponent for SheetModel {
    type Init = PathBuf;
    type Input = SheetModelInput;
    type Output = SheetPressedMessage;

    view! {
        #[root]
        gtk::Box {
            set_orientation: gtk::Orientation::Vertical,
            append = &gtk::Button {
                set_label: &format!("{}", model.path.file_name().unwrap().to_string_lossy()),
                set_halign: gtk::Align::Start,
                set_margin_all: 10,
                connect_clicked[sender] => move |_| sender.input(SheetModelInput::OnClicked),
            }
        }
    }

    fn init(
        path: Self::Init,
        root: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let model = SheetModel { path };
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, sender: ComponentSender<Self>) {
        match message {
            SheetModelInput::OnClicked => sender
                .output(SheetPressedMessage::SheetPressed(self.path.clone()))
                .unwrap(),
        }
    }
}

pub struct SheetListingModel {
    _sheet_models: Vec<Controller<SheetModel>>,
}

#[relm4::component(pub)]
impl SimpleComponent for SheetListingModel {
    type Init = PathBuf;
    type Input = ();
    type Output = SheetPressedMessage;

    view! {
        #[root]
        gtk::Box {
            set_orientation: gtk::Orientation::Vertical,
        }
    }

    fn init(
        dir: Self::Init,
        root: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let mut sheet_models = Vec::new();

        for entry in WalkDir::new(dir)
            .into_iter()
            .filter_map(|e| e.ok())
            .filter(|file| file.file_type().is_file())
            .map(|file| file.into_path())
            .filter(|path| {
                path.extension()
                    .map(|s| s.to_string_lossy().to_ascii_lowercase() == "pdf")
                    .unwrap_or(false)
            })
        {
            let sheet_model = SheetModel::builder()
                .launch(entry)
                .forward(sender.output_sender(), |m| m);
            root.append(sheet_model.widget());
            sheet_models.push(sheet_model);
        }

        let model = SheetListingModel {
            _sheet_models: sheet_models,
        };
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }
}
